"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const driver_1 = require("./driver");
const logic_1 = require("./logic");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const model_1 = require("./model");
class EcrModule {
    constructor(eeDriver, oscDriver, persistence) {
        this.oscDriver = oscDriver;
        this.persistence = persistence;
        this.pulse = rxjs_1.interval(1111);
        this.driver = new driver_1.EcrDriver(eeDriver, this.pulse);
        this.model = new model_1.EcrModel();
        this.logic = new logic_1.EcrLogic(this.driver, this.model);
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.persistence.init(this.model);
            this.oscDriver.inbox.filter(m => m.address.startsWith('/d/repairs')).subscribe(message => {
                const addressArr = message.address.split('/');
                if (addressArr.length === 5) {
                    const command = addressArr[4];
                    const systemName = addressArr[3].toLowerCase();
                    const s2 = logic_1.lowercaseInfraSystemNames.indexOf(systemName);
                    if (~s2) {
                        //   console.log('*********MSG:', message.address);
                        switch (command) {
                            case 'error':
                                this.logic.setError(s2);
                                break;
                            case 'start-up':
                                this.logic.startupSwitchBoard(s2);
                                break;
                            case 'shut-down':
                                this.logic.shutdownSwitchBoard(s2);
                                break;
                            case 'overload-threshold':
                                const overloadThreshold = message.args[0].value;
                                this.logic.setOverloadThreshold(s2, overloadThreshold);
                                break;
                            case 'load':
                                const overload = message.args[0].value;
                                this.logic.setOverload(s2, overload);
                                break;
                            default:
                                console.error('unknown command', command);
                        }
                    }
                    else {
                        console.error('unknown system', systemName);
                    }
                }
                else {
                    console.error('maleformed address', message.address);
                }
            });
            this.broadcastSystemsState();
            yield this.driver.init();
            yield this.logic.init();
        });
    }
    broadcastSystemsState() {
        this.subscription = this.pulse.pipe(operators_1.switchMap(_ => {
            const result = [];
            /*
            for (let s1 = 0; s1 < ESystem.COUNT; s1++) {
                const system1 = this.logic.getPrimarySystemStatus(s1);
                result.push({
                    address: `/d/repairs/${ESystem[s1]}/repair-rate`,
                    args: {type: 'f', value: system1.repairRate}
                }, {
                    address: `/d/repairs/${ESystem[s1]}/heat-rate`,
                    args: {type: 'f', value: system1.heatRate}
                }, {
                    address: `/d/repairs/${ESystem[s1]}/max-power`,
                    args: {type: 'f', value: system1.maxPower}
                });
            }
            */
            for (let s2 = 0; s2 < model_1.ESwitchBoard.COUNT; s2++) {
                const system2 = this.logic.getSwitchBoardStatus(s2);
                result.push({
                    address: `/d/repairs/${model_1.ESwitchBoard[s2]}/is-error`,
                    args: { type: 'i', value: system2.isError ? 1 : 0 }
                }, {
                    address: `/d/repairs/${model_1.ESwitchBoard[s2]}/is-online`,
                    args: { type: 'i', value: system2.isOnline ? 1 : 0 }
                }, {
                    address: `/d/repairs/${model_1.ESwitchBoard[s2]}/load`,
                    args: { type: 'f', value: system2.overload }
                }, {
                    address: `/d/repairs/${model_1.ESwitchBoard[s2]}/overload-threshold`,
                    args: { type: 'f', value: system2.overloadErrorThreshold }
                });
            }
            return result;
        })).subscribe(this.oscDriver.outbox);
    }
    beginRepair(id) {
        this.logic.startRepairingPrimarySystem(id);
    }
    stopRepair() {
        this.logic.stopRepairingPrimarySystem();
    }
    destroy() {
        this.subscription.unsubscribe();
        this.logic.destroy();
    }
}
exports.EcrModule = EcrModule;
//# sourceMappingURL=index.js.map