"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const model_1 = require("./model");
const empty_epsilon_js_1 = require("empty-epsilon-js");
const timing_1 = require("../core/timing");
exports.InfraSystemNames = Array.from(Array(model_1.ESwitchBoard.COUNT)).map((_, i) => model_1.ESwitchBoard[i]);
exports.lowercaseInfraSystemNames = exports.InfraSystemNames.map((_, i) => model_1.ESwitchBoard[i].toLowerCase());
class EcrLogic {
    constructor(driver, model) {
        this.driver = driver;
        this.model = model;
    }
    get repairing() {
        return this.model.repairing;
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield Promise.all(Array.from(Array(empty_epsilon_js_1.ESystem.COUNT)).map((_, s1) => this.updateMaxPower(s1)));
            if (this.disposer === undefined) {
                // initiate game loop
                const ticker = timing_1.setTimedInterval(this.tick.bind(this), EcrLogic.tickInterval);
                // register for power updates
                const subscription = this.driver.powerUpdates.subscribe(msg => this.model.primarySystems[msg.system].power = msg.power);
                this.disposer = () => {
                    clearInterval(ticker);
                    subscription.unsubscribe();
                };
            }
        });
    }
    /**
     * the "game loop" logic for this module
     * @param {number} delta time (double precision of milliseconds) since lst tick
     */
    tick(delta) {
        for (let s1 = 0; s1 < empty_epsilon_js_1.ESystem.COUNT; s1++) {
            if (s1 !== empty_epsilon_js_1.ESystem.Reactor) {
                const system1 = this.model.primarySystems[s1];
                const overPowerFactor = system1.normalizedOverPower;
                if (overPowerFactor) {
                    system1.switchboards.forEach(system2 => this.addOverloadToSwitchBoard(system2.id, delta * model_1.SwitchBoard.overloadPerMillisecond * overPowerFactor / system1.switchboards.length));
                }
            }
        }
    }
    destroy() {
        if (this.disposer !== undefined) {
            this.disposer();
            delete this.disposer;
        }
    }
    addOverloadToSwitchBoard(id, overload) {
        const system2 = this.model.switchBoards[id];
        system2.addOverload(overload);
        if (system2.overload > system2.overloadErrorThreshold) {
            this.setError(system2.id);
        }
    }
    getPrimarySystemStatus(id) {
        return this.model.primarySystems[id];
    }
    getSwitchBoardStatus(id) {
        return this.model.switchBoards[id];
    }
    setOverload(id, value) {
        const system2 = this.model.switchBoards[id];
        system2.overload = value;
        if (system2.overload > system2.overloadErrorThreshold) {
            this.setError(system2.id);
        }
    }
    setOverloadThreshold(id, value) {
        const system2 = this.model.switchBoards[id];
        system2.overloadErrorThreshold = value;
        if (system2.overload > system2.overloadErrorThreshold) {
            this.setError(system2.id);
        }
    }
    setError(id) {
        this.model.switchBoards[id].setError();
        this.model.switchBoards[id].supportedSystems.forEach(sys1 => {
            this.updateHeatRate(sys1.id);
            this.updateMaxPower(sys1.id);
            this.updateRepairRate(sys1.id);
        });
    }
    shutdownSwitchBoard(id) {
        this.model.switchBoards[id].shutdown();
        this.model.switchBoards[id].supportedSystems.forEach(sys1 => {
            this.updateHeatRate(sys1.id);
            this.updateMaxPower(sys1.id);
            this.updateRepairRate(sys1.id);
        });
    }
    startupSwitchBoard(id) {
        this.model.switchBoards[id].startup();
        this.model.switchBoards[id].supportedSystems.forEach(sys1 => {
            this.updateHeatRate(sys1.id);
            this.updateMaxPower(sys1.id);
            this.updateRepairRate(sys1.id);
        });
    }
    startRepairingPrimarySystem(id) {
        if (this.model.repairing === null) {
            this.model.repairing = id;
            this.updateRepairRate(id);
        }
        else if (this.model.repairing !== id) {
            throw new Error(`currently repairing ${empty_epsilon_js_1.ESystem[this.model.repairing]}`);
        }
    }
    stopRepairingPrimarySystem() {
        if (this.model.repairing !== null) {
            this.driver.setRepairRate(this.model.repairing, 0);
            this.model.repairing = null;
        }
    }
    updateHeatRate(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.driver.setHeatRate(id, this.model.primarySystems[id].heatRate);
        });
    }
    updateMaxPower(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.driver.setMaxPower(id, this.model.primarySystems[id].maxPower);
        });
    }
    updateRepairRate(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.model.repairing === id) {
                yield this.driver.setRepairRate(id, this.model.primarySystems[id].repairRate);
            }
        });
    }
}
exports.EcrLogic = EcrLogic;
EcrLogic.tickInterval = 10;
//# sourceMappingURL=logic.js.map