"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const empty_epsilon_js_1 = require("empty-epsilon-js");
class PrimarySystem {
    constructor(id, switchboards) {
        this.id = id;
        this.switchboards = switchboards;
        this.power = 0;
    }
    get repairRate() {
        const onlineNoErrorSystems = this.switchboards.filter(sys => sys.isOnline && !sys.isError).length;
        switch (onlineNoErrorSystems) {
            case 0:
                return PrimarySystem.repairRate * 0.2;
            case 1:
                return PrimarySystem.repairRate * 0.5;
            default:
                return PrimarySystem.repairRate;
        }
    }
    get heatRate() {
        const onlineNoErrorSystems = this.switchboards.filter(sys => sys.isOnline && !sys.isError).length;
        const errorSystems = this.switchboards.filter(sys => sys.isError).length;
        if (errorSystems && (errorSystems >= onlineNoErrorSystems)) {
            return PrimarySystem.heatOnErrorRate;
        }
        else {
            return 0;
        }
        // if (this.switchboards.every(sys => !sys.isError)) {
        //     return 0;
        // } else {
        //     return PrimarySystem.heatOnErrorRate;
        // }
    }
    get maxPower() {
        let onlineSystems = this.switchboards.filter(sys => sys.isOnline).length;
        if (onlineSystems === this.switchboards.length) {
            return PrimarySystem.maxOverPower;
        }
        else {
            return PrimarySystem.maxSupportedPower * onlineSystems / this.switchboards.length;
        }
    }
    /**
     * between 0 and 2,  the factor of over-power
     */
    get normalizedOverPower() {
        return Math.max(0, (this.power - PrimarySystem.maxSupportedPower) / PrimarySystem.maxSupportedPower);
    }
}
exports.PrimarySystem = PrimarySystem;
PrimarySystem.maxOverPower = 3.0;
PrimarySystem.maxSupportedPower = 1.0;
PrimarySystem.heatOnErrorRate = 1.0;
PrimarySystem.repairRate = 1.0;
var ESwitchBoard;
(function (ESwitchBoard) {
    ESwitchBoard[ESwitchBoard["None"] = -1] = "None";
    ESwitchBoard[ESwitchBoard["A1"] = 0] = "A1";
    ESwitchBoard[ESwitchBoard["A2"] = 1] = "A2";
    ESwitchBoard[ESwitchBoard["A3"] = 2] = "A3";
    ESwitchBoard[ESwitchBoard["B1"] = 3] = "B1";
    ESwitchBoard[ESwitchBoard["B2"] = 4] = "B2";
    ESwitchBoard[ESwitchBoard["B3"] = 5] = "B3";
    ESwitchBoard[ESwitchBoard["COUNT"] = 6] = "COUNT";
})(ESwitchBoard = exports.ESwitchBoard || (exports.ESwitchBoard = {}));
class SwitchBoard {
    constructor(id) {
        this.id = id;
        this.supportedSystems = [];
        this.shutdown();
        this.startup();
    }
    get name() {
        return ESwitchBoard[this.id];
    }
    addOverload(delta) {
        if (this.isOnline) {
            this.overload = Math.min(SwitchBoard.maxOverload, this.overload + delta);
        }
    }
    shutdown() {
        this.isOnline = false;
        this.isError = false;
        this.overload = 0;
        this.overloadErrorThreshold = Math.max(Math.random() * SwitchBoard.maxOverload, SwitchBoard.overloadPerMillisecond * 1000);
    }
    startup() {
        this.isOnline = true;
    }
    setError() {
        if (this.isOnline) {
            this.isError = true;
        }
    }
    toJSON() {
        return {
            name: this.name,
            isError: this.isError,
            isOnline: this.isOnline,
            overload: this.overload,
            overloadErrorThreshold: this.overloadErrorThreshold,
        };
    }
    fromJSON(state) {
        this.isError = state.isError;
        this.isOnline = state.isOnline;
        this.overload = state.overload;
        this.overloadErrorThreshold = state.overloadErrorThreshold;
    }
}
exports.SwitchBoard = SwitchBoard;
// say it takes ~3 minutes of ~200% power to cause an error on average
// so it should take twice to reach maximum overload (1).
// so overload per milli :
// 1 / 2 * 3 * 60 * 1000  = 1 / 180000 ~= 0.0000055
SwitchBoard.overloadPerMillisecond = 0.0000055;
SwitchBoard.maxOverload = 1;
// const allSystems = [ESystem.Reactor, ESystem.BeamWeapons, ESystem.MissileSystem,
//     ESystem.Maneuver, ESystem.Impulse, ESystem.Warp,
//     ESystem.JumpDrive, ESystem.FrontShield, ESystem.RearShield, ESystem.Docks, ESystem.Drones];
class EcrModel {
    constructor() {
        // static readonly switchboardstMap: { [switchId: number]: ESystem[] } = {
        //     [ESwitchBoard.A1]: allSystems,
        //     [ESwitchBoard.A2]: allSystems,
        //     [ESwitchBoard.A3]: [],
        //     [ESwitchBoard.B1]: [],
        //     [ESwitchBoard.B2]: [],
        //     [ESwitchBoard.B3]: [],
        // };
        this.primarySystems = {};
        this.switchBoards = {};
        this.repairing = null;
        const connections = {};
        for (let s1 = 0; s1 < empty_epsilon_js_1.ESystem.COUNT; s1++) {
            connections[s1] = [];
            this.primarySystems[s1] = new PrimarySystem(s1, connections[s1]);
        }
        for (let s2 = 0; s2 < ESwitchBoard.COUNT; s2++) {
            let sys2 = new SwitchBoard(s2);
            for (let s1 of EcrModel.switchboardstMap[s2]) {
                const system1 = this.primarySystems[s1];
                connections[s1].push(sys2);
                sys2.supportedSystems.push(system1);
            }
            this.switchBoards[s2] = sys2;
        }
    }
    toJSON() {
        return {
            repairing: this.repairing,
            switchBoards: Array.from(Array(ESwitchBoard.COUNT)).map((_, sb) => this.switchBoards[sb].toJSON()),
        };
    }
    fromJSON(state) {
        this.repairing = state.repairing;
        for (let sb = 0; sb < ESwitchBoard.COUNT; sb++) {
            if (this.switchBoards[sb].name === state.switchBoards[sb].name) {
                this.switchBoards[sb].fromJSON(state.switchBoards[sb]);
            }
            else {
                throw new Error(`at index ${sb}, object ${this.switchBoards[sb].name} but state name is ${state.switchBoards[sb].name}`);
            }
        }
    }
}
exports.EcrModel = EcrModel;
EcrModel.switchboardstMap = {
    [ESwitchBoard.A1]: [empty_epsilon_js_1.ESystem.MissileSystem, empty_epsilon_js_1.ESystem.FrontShield, empty_epsilon_js_1.ESystem.Reactor],
    [ESwitchBoard.A2]: [empty_epsilon_js_1.ESystem.BeamWeapons, empty_epsilon_js_1.ESystem.Warp, empty_epsilon_js_1.ESystem.Reactor],
    [ESwitchBoard.A3]: [empty_epsilon_js_1.ESystem.Maneuver, empty_epsilon_js_1.ESystem.Impulse, empty_epsilon_js_1.ESystem.RearShield, empty_epsilon_js_1.ESystem.Reactor],
    [ESwitchBoard.B1]: [empty_epsilon_js_1.ESystem.FrontShield, empty_epsilon_js_1.ESystem.Impulse, empty_epsilon_js_1.ESystem.BeamWeapons, empty_epsilon_js_1.ESystem.RearShield],
    [ESwitchBoard.B2]: [],
    [ESwitchBoard.B3]: [empty_epsilon_js_1.ESystem.MissileSystem, empty_epsilon_js_1.ESystem.Maneuver, empty_epsilon_js_1.ESystem.Warp, empty_epsilon_js_1.ESystem.Reactor],
};
//# sourceMappingURL=model.js.map