"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kissfs_1 = require("kissfs");
const clearence_1 = require("./clearence");
const path_1 = require("path");
const fm = require("front-matter");
var Status;
(function (Status) {
    Status[Status["DRAFT"] = 0] = "DRAFT";
    Status[Status["QUERY"] = 1] = "QUERY";
    Status[Status["ENTRY"] = 2] = "ENTRY";
})(Status = exports.Status || (exports.Status = {}));
class Entry {
    constructor(path, meta, content) {
        this.path = path;
        this.meta = meta;
        this.content = content;
        if (this.meta.status === undefined) {
            this.meta.status = Status.DRAFT;
        }
        if (this.meta.securityClass === undefined) {
            this.meta.securityClass = clearence_1.ExcaliburSecClass.CONFIDENTIAL;
        }
        if (this.meta.name === undefined) {
            this.meta.name = "unknown";
        }
        this.meta.name = this.meta.name.substr(0, 1).toUpperCase() + this.meta.name.substr(1);
    }
    static parse(rawContent, path) {
        const parsed = fm(rawContent.trim());
        return new Entry(path, parsed.attributes, parsed.body);
    }
    toString() {
        return `---
status : ${this.meta.status}
securityClass : ${this.meta.securityClass}
name : '${this.meta.name}'
---
${this.content}`;
    }
}
exports.Entry = Entry;
const DEFAULT_OPTIONS = {
    entriesPath: 'entries',
    queriesPath: 'queries',
    queriesPrefix: 'query',
    generateQueryBody() {
        return `This entry was created due to an anonymous query at ${Date.now()}`;
    }
};
class Entries {
    constructor(fs, options = DEFAULT_OPTIONS) {
        this.fs = fs;
        this.options = options;
        this.hasEntries = true;
        this.entries = new Map();
        this.fileHandler = (e) => {
            // Handle all new files found in the folder
            if (e.fullPath.startsWith(this.options.entriesPath) || e.fullPath.startsWith(this.options.queriesPath)) {
                let entry = Entry.parse(e.newContent, e.fullPath);
                if (entry.meta.status === Status.ENTRY) {
                    this.entries.set(e.fullPath, entry);
                }
                else {
                    this.entries.delete(e.fullPath);
                }
            }
        };
        this.fileDeleteHandler = (e) => {
            if (e.fullPath.startsWith(this.options.entriesPath) || e.fullPath.startsWith(this.options.queriesPath)) {
                this.entries.delete(e.fullPath);
            }
        };
        this.scanQueries = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const queries = yield this.fs.loadDirectoryChildren(this.options.queriesPath);
                yield queries.filter(kissfs_1.isFile)
                    // load file
                    .map((file) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    try {
                        file.content = yield this.fs.loadTextFile(file.fullPath);
                        const entry = Entry.parse(file.content, file.fullPath);
                        if (entry.meta.status === Status.ENTRY) {
                            yield this.copyToEntries(entry);
                        }
                    }
                    catch (e) {
                    }
                }))
                    // wait for all loading
                    .reduce((r, p) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield r;
                    yield p;
                }), {});
            }
            finally {
                this.watchdog = setTimeout(this.scanQueries, 1000);
            }
        });
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.fs.events.on('fileChanged', this.fileHandler);
            this.fs.events.on('fileCreated', this.fileHandler);
            this.fs.events.on('fileDeleted', this.fileDeleteHandler);
            yield this.scanEntries();
            yield this.scanQueries();
        });
    }
    scanEntries() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let fsItems;
            try {
                fsItems = yield this.fs.loadDirectoryChildren(this.options.entriesPath);
                this.hasEntries = true;
            }
            catch (e) {
                console.warn(`Path ${this.options.entriesPath} does not exist. entries will not be scanned`);
                this.hasEntries = false;
                return;
            }
            // process all files
            yield fsItems.filter(kissfs_1.isFile)
                // load file
                .map((file) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const content = yield this.fs.loadTextFile(file.fullPath);
                this.fileHandler({ type: "fileChanged", fullPath: file.fullPath, newContent: content });
            }))
                // wait for all loading
                .reduce((r, p) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield r;
                yield p;
            }), {});
        });
    }
    destroy() {
        this.fs.events.off('fileChanged', this.fileHandler);
        this.fs.events.off('fileCreated', this.fileHandler);
        this.fs.events.off('fileDeleted', this.fileDeleteHandler);
        clearTimeout(this.watchdog);
    }
    open(user, search) {
        let entry = [...this.entries.values()].find(e => user.isPermitted(e.meta.securityClass) && e.meta.name.toLowerCase().includes(search.toLowerCase()));
        return entry && entry.content;
    }
    list() {
        return [...this.entries.values()]
            .map(e => e.meta.name)
            .sort();
    }
    query(user, search) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let queryFileName = `${this.options.queriesPrefix}_${Date.now().toString()}.md`;
            let newPath = path_1.join(this.options.queriesPath, queryFileName);
            let queryEntry = new Entry(newPath, {
                status: Status.QUERY,
                securityClass: user.excaliburClearance,
                name: search
            }, this.options.generateQueryBody(search));
            yield this.fs.saveFile(newPath, queryEntry.toString());
            const entry = yield new Promise((resolve, reject) => {
                const cleanup = () => {
                    this.fs.events.removeListener('fileDeleted', listener);
                    this.fs.events.removeListener('fileChanged', listener);
                };
                const listener = (e) => {
                    if (path_1.normalize(e.fullPath) === path_1.normalize(newPath)) {
                        switch (e.type) {
                            case 'fileChanged':
                                let entry = Entry.parse(e.newContent, e.fullPath);
                                if (entry.meta.status === Status.ENTRY) {
                                    cleanup();
                                    resolve(entry);
                                }
                                break;
                            case 'fileDeleted':
                                cleanup();
                                reject(new Error("Bad command or file name"));
                                break;
                        }
                    }
                };
                this.fs.events.on('fileChanged', listener);
                this.fs.events.on('fileDeleted', listener);
            });
            return this.open(user, entry.meta.name);
        });
    }
    copyToEntries(entry) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.hasEntries) {
                let entriesPath = path_1.join(this.options.entriesPath, path_1.basename(entry.path));
                yield this.fs.saveFile(entriesPath, entry.toString());
            }
            yield this.fs.deleteFile(entry.path);
        });
    }
}
exports.Entries = Entries;
//# sourceMappingURL=entries.js.map