"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kissfs_1 = require("kissfs");
const empty_epsilon_js_1 = require("empty-epsilon-js");
const open_epsilon_1 = require("open-epsilon");
const index_1 = require("./ecr/index");
const persistency_1 = require("./core/persistency");
const rpc_1 = require("./ecr/rpc");
exports.FILE_PATH = 'game-monitor.json';
function getMonitoredAddresses(fs) {
    const result = [];
    function handleFileContent(fileContent) {
        try {
            const addresses = JSON.parse(fileContent.toLowerCase());
            result.splice(0, result.length, ...addresses);
        }
        catch (e) {
            console.error(`failed parsing ${exports.FILE_PATH} : ${fileContent}`);
        }
    }
    fs.events.on('fileChanged', ({ newContent, fullPath }) => {
        if (fullPath === exports.FILE_PATH) {
            handleFileContent(newContent);
        }
    });
    fs.loadTextFile(exports.FILE_PATH).then(handleFileContent);
    return result;
}
exports.getMonitoredAddresses = getMonitoredAddresses;
class SimulatorServices {
    constructor(options, fs) {
        this.fs = fs;
        this.oscDriver = new open_epsilon_1.OscUdpDriver(options.oscOptions);
        this.eeDriver = new empty_epsilon_js_1.HttpDriver(options.eeAddress);
        this.ecrModule = new index_1.EcrModule(this.eeDriver, this.oscDriver, new persistency_1.Persistence('ECR', fs, 'ecr-state.json'));
        this.rpcServer = rpc_1.expose(this.ecrModule, { hostname: '0.0.0.0', port: options.rpcPort });
        this.openEpsilon = new open_epsilon_1.OpenEpsilon(this.eeDriver, this.oscDriver);
        this.openEpsilon.monitoredAddresses = getMonitoredAddresses(fs);
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.oscDriver.open();
            yield this.ecrModule.init();
            this.openEpsilon.init();
        });
    }
    close() {
        this.openEpsilon.destroy();
        this.oscDriver.close();
        this.ecrModule.destroy();
        this.eeDriver.close();
        this.rpcServer.close();
    }
}
exports.SimulatorServices = SimulatorServices;
process.on('uncaughtException', function (err) {
    console.error(err.message);
    console.error(err.stack);
});
function main(optionsArg) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const options = Object.assign({}, exports.DEFAULT_OPTIONS, optionsArg);
        const fs = yield (new kissfs_1.LocalFileSystem(optionsArg.resources)).init();
        const simulatorServices = new SimulatorServices(options, fs);
        const exit = simulatorServices.close.bind(simulatorServices);
        process.once('exit', exit);
        process.once('SIGINT', exit.bind(null, true, 2));
        process.once('SIGTERM', exit.bind(null, true, 15));
        yield simulatorServices.init();
    });
}
exports.main = main;
exports.DEFAULT_OPTIONS = {
    oscOptions: {
        localAddress: "0.0.0.0",
        localPort: 57121,
        remotePort: 57122
    }
};
//# sourceMappingURL=simulation.js.map