"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kissfs_1 = require("kissfs");
const entries_1 = require("../../src/excalibur/entries");
const users_1 = require("../../src/session/users");
const clearence_1 = require("../../src/excalibur/clearence");
const chai_1 = require("chai");
const empty_epsilon_js_1 = require("empty-epsilon-js");
function makeUser(clearence) {
    const result = new users_1.User({
        gender: "ADVANCED",
        name: "jane joe",
        key: 'j',
        excaliburClearance: "CONFIDENTIAL"
    });
    result.excaliburClearance = clearence;
    return result;
}
function makeEntryFile(status, securityClass, name, content) {
    return `---
status : ${status}
securityClass : ${securityClass}
name : ${name}
---
${content}`;
}
function makeEntriesWithPreExistingFile(fileContent) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fs = new kissfs_1.MemoryFileSystem();
        yield fs.ensureDirectory('entries');
        yield fs.ensureDirectory('queries');
        yield fs.saveFile('entries/zagzag.md', fileContent);
        const entries = new entries_1.Entries(fs);
        after(() => entries.destroy());
        yield entries.init();
        return entries;
    });
}
function makeEntrieandFileSystem() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fs = new kissfs_1.MemoryFileSystem();
        yield fs.ensureDirectory('entries');
        yield fs.ensureDirectory('queries');
        const entries = new entries_1.Entries(fs);
        after(() => entries.destroy());
        yield entries.init();
        return { fs, entries };
    });
}
describe.skip('excalibur module', () => {
    describe('init', () => {
        it('on empty file sytsem - throws', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            const fs = new kissfs_1.MemoryFileSystem();
            const entries = new entries_1.Entries(fs);
            try {
                yield entries.init();
                chai_1.expect.fail(null, 'Error', 'expected init to throw');
            }
            catch (e) {
            }
        }));
        it('on non-empty file sytsem - passes', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            const fs = new kissfs_1.MemoryFileSystem();
            yield fs.ensureDirectory('entries');
            yield fs.ensureDirectory('queries');
            const entries = new entries_1.Entries(fs);
            yield entries.init();
        }));
    });
    describe('list', () => {
        describe('pre-existing file', () => {
            it('shows only entries', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const { entries, fs } = yield makeEntrieandFileSystem();
                yield fs.saveFile('entries/foo1.md', makeEntryFile(entries_1.Status.DRAFT, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo1', 'bar'));
                yield fs.saveFile('entries/foo2.md', makeEntryFile(entries_1.Status.QUERY, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo2', 'bar'));
                yield fs.saveFile('entries/foo3.md', makeEntryFile(entries_1.Status.ENTRY, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo3', 'bar'));
                yield fs.saveFile('entries/foo4.md', makeEntryFile(entries_1.Status.ENTRY, clearence_1.ExcaliburSecClass.SECRET, 'foo4', 'bar'));
                const result = entries.list();
                chai_1.expect(result.map(s => s.trim()).sort()).to.eql(['Foo3', 'Foo4']);
            }));
        });
    });
    describe('open', () => {
        describe('pre-existing file', () => {
            it('does not find draft file', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const entries = yield makeEntriesWithPreExistingFile(makeEntryFile(entries_1.Status.DRAFT, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo', 'bar'));
                const result = entries.open(makeUser(clearence_1.ExcaliburSecClass.CONFIDENTIAL), 'foo');
                chai_1.expect(result).to.not.be.ok;
            }));
            it('does not find query file', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const entries = yield makeEntriesWithPreExistingFile(makeEntryFile(entries_1.Status.QUERY, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo', 'bar'));
                const result = entries.open(makeUser(clearence_1.ExcaliburSecClass.CONFIDENTIAL), 'foo');
                chai_1.expect(result).to.not.be.ok;
            }));
            it('finds entry file', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const entries = yield makeEntriesWithPreExistingFile(makeEntryFile(entries_1.Status.ENTRY, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo', 'bar'));
                const result = entries.open(makeUser(clearence_1.ExcaliburSecClass.CONFIDENTIAL), 'foo');
                chai_1.expect(result).to.eql('bar');
            }));
            it('finds entry with whitespace before metadata', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const entries = yield makeEntriesWithPreExistingFile('\n  \t  ' + makeEntryFile(entries_1.Status.ENTRY, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'foo', 'bar'));
                const result = entries.open(makeUser(clearence_1.ExcaliburSecClass.CONFIDENTIAL), 'foo');
                chai_1.expect(result).to.eql('bar');
            }));
            it('finds entry by case-insensitive name', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const entries = yield makeEntriesWithPreExistingFile(makeEntryFile(entries_1.Status.ENTRY, clearence_1.ExcaliburSecClass.CONFIDENTIAL, 'FoO', 'bar'));
                const result = entries.open(makeUser(clearence_1.ExcaliburSecClass.CONFIDENTIAL), 'fOo');
                chai_1.expect(result).to.eql('bar');
            }));
        });
    });
    describe('query', () => {
        function loadSingleEntry(fs, path) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const children = yield fs.loadDirectoryChildren(path);
                chai_1.expect(children).to.have.length(1);
                const file = children[0];
                file.content = yield fs.loadTextFile(file.fullPath);
                return entries_1.Entry.parse(file.content, file.fullPath);
            });
        }
        function query(entries, fs) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const result = entries.query(makeUser(clearence_1.ExcaliburSecClass.CONFIDENTIAL), 'foo');
                yield new Promise(r => setTimeout(r, 1));
                const entry = yield loadSingleEntry(fs, 'queries');
                return { result, entry };
            });
        }
        it('finds entry file', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            const { entries, fs } = yield makeEntrieandFileSystem();
            chai_1.expect(yield fs.loadDirectoryChildren('queries')).to.have.length(0);
            const { result, entry } = yield query(entries, fs);
            chai_1.expect(entry.meta.status).to.eql(entries_1.Status.QUERY);
            chai_1.expect(entry.meta.securityClass).to.eql(clearence_1.ExcaliburSecClass.CONFIDENTIAL);
            entry.content = 'bar123';
            entry.meta.status = entries_1.Status.ENTRY;
            yield fs.saveFile(entry.path, entry.toString());
            chai_1.expect(yield result).to.eql('bar123');
        }));
        describe('afterwards', () => {
            let entries;
            let entry;
            let fs;
            beforeEach('finds entry file', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const enf = yield makeEntrieandFileSystem();
                entries = enf.entries;
                fs = enf.fs;
                entry = (yield query(entries, fs)).entry;
                entry.content = 'bar123';
                entry.meta.status = entries_1.Status.ENTRY;
                yield fs.saveFile(entry.path, entry.toString());
            }));
            it('entry moves from queries to entries folder', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                yield empty_epsilon_js_1.retry(() => tslib_1.__awaiter(void 0, void 0, void 0, function* () { return chai_1.expect(yield fs.loadDirectoryChildren('queries')).to.have.length(0); }), {
                    interval: 100,
                    timeout: 5 * 1000
                });
                chai_1.expect(yield fs.loadDirectoryChildren('entries')).to.have.length(1);
                const entryInEntries = yield loadSingleEntry(fs, 'entries');
                chai_1.expect(entryInEntries.meta).to.eql(entry.meta);
                chai_1.expect(entryInEntries.content).to.eql(entry.content);
            }));
            it('entry immediately appears in list', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const result = entries.list();
                chai_1.expect(result).to.eql(['Foo']);
            }));
        });
    });
});
//# sourceMappingURL=excalibur.spec.js.map