"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const driver_1 = require("../../src/ecr/driver");
const rxjs_1 = require("rxjs");
const empty_epsilon_js_1 = require("empty-epsilon-js");
const config_1 = require("../test-kit/config");
const chai_1 = require("chai");
const model_1 = require("../../src/ecr/model");
const test_kit_1 = require("./test-kit");
describe('EcrDriver', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    let pulser;
    let httpDriver;
    let repairDriver;
    empty_epsilon_js_1.eeTestServerLifecycle(config_1.default);
    beforeEach(() => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        let pulse = new rxjs_1.Observable((s) => pulser = s);
        httpDriver = new empty_epsilon_js_1.HttpDriver(config_1.default.serverAddress);
        repairDriver = new driver_1.EcrDriver(httpDriver, pulse);
        yield repairDriver.init();
    }));
    function powerUpdate() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const updates = [];
            const subscription = repairDriver.powerUpdates.subscribe(update => updates.push(update));
            pulser.next();
            yield empty_epsilon_js_1.retry(() => {
                chai_1.expect(updates.length).to.equal(empty_epsilon_js_1.ESystem.COUNT);
            }, { interval: 30, timeout: 1500 });
            subscription.unsubscribe();
            return updates;
        });
    }
    it('powerUpdates', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const updates = yield powerUpdate();
        const expected = Array.from(Array(empty_epsilon_js_1.ESystem.COUNT)).map((_, i) => ({
            system: i,
            power: model_1.PrimarySystem.maxSupportedPower
        }));
        chai_1.expect(updates).to.eql(expected);
    }));
    it('setMaxPower', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const MAX_POWER_TEST_VAL = 0.3;
        const SYSTEM = empty_epsilon_js_1.ESystem.MissileSystem;
        const sysPredicate = s => s.system === SYSTEM;
        yield repairDriver.setMaxPower(SYSTEM, MAX_POWER_TEST_VAL);
        const updates = yield powerUpdate();
        const expected = Array.from(Array(empty_epsilon_js_1.ESystem.COUNT)).map((_, i) => ({
            system: i,
            power: i === SYSTEM ? MAX_POWER_TEST_VAL : model_1.PrimarySystem.maxSupportedPower
        }));
        chai_1.expect(updates.filter(i => !sysPredicate(i))).to.eql(expected.filter(i => !sysPredicate(i)));
        chai_1.expect(updates.find(sysPredicate).power).to.be.gt(MAX_POWER_TEST_VAL - 0.01);
        chai_1.expect(updates.find(sysPredicate).power).to.be.lt(MAX_POWER_TEST_VAL + 0.01);
    }));
    it('setRepairRate', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(20 * 1000);
            const graceFactor = 0.5;
            const SYSTEM = empty_epsilon_js_1.ESystem.MissileSystem;
            yield httpDriver.command(`getPlayerShip(-1):setSystemHealth('${empty_epsilon_js_1.ESystem[SYSTEM]}', {0})`, ['0.1']);
            const scale = 10000;
            const repairRate = 0.5;
            yield repairDriver.setRepairRate(SYSTEM, repairRate);
            const expected = scale * repairRate * driver_1.repair_per_second / 1000;
            chai_1.expect(yield test_kit_1.getLinearDeriviation(() => httpDriver.query(`getPlayerShip(-1):getSystemHealth('${empty_epsilon_js_1.ESystem[SYSTEM]}') * ${scale}`), {
                iterations: 4,
                graceFactor,
                tickInterval: 10
            })).to.be.approximately(expected, expected * graceFactor);
        });
    });
    it('setHeatRate', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(20 * 1000);
            const graceFactor = 0.2;
            const SYSTEM = empty_epsilon_js_1.ESystem.MissileSystem;
            const scale = 10000;
            const heatRate = 0.5;
            yield repairDriver.setHeatRate(SYSTEM, heatRate);
            //    await new Promise(res => setTimeout(res, 100));
            const expected = scale * heatRate * driver_1.heat_per_second / 1000;
            chai_1.expect(yield test_kit_1.getLinearDeriviation(() => httpDriver.query(`getPlayerShip(-1):getSystemHeat('${empty_epsilon_js_1.ESystem[SYSTEM]}') * ${scale}`), {
                iterations: 4,
                graceFactor,
                tickInterval: 10
            })).to.be.approximately(expected, expected * graceFactor);
        });
    });
}));
//# sourceMappingURL=driver.spec.js.map