"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const logic_1 = require("../../src/ecr/logic");
const empty_epsilon_js_1 = require("empty-epsilon-js");
const sinon_1 = require("sinon");
const model_1 = require("../../src/ecr/model");
const test_kit_1 = require("./test-kit");
const Observable_1 = require("rxjs/Observable");
const graceFactor = 0.3;
describe('repair module', () => {
    const sideEffects = {
        setRepairRate: sinon_1.spy(),
        setHeatRate: sinon_1.spy(),
        setMaxPower: sinon_1.spy(),
        powerInput: undefined,
        powerUpdates: undefined
    };
    sideEffects.powerUpdates = new Observable_1.Observable(subscriber => sideEffects.powerInput = subscriber);
    let repair;
    beforeEach('init module', () => {
        repair = new logic_1.EcrLogic(sideEffects, new model_1.EcrModel());
        repair.init();
        for (let s2 = 0; s2 < model_1.ESwitchBoard.COUNT; s2++) {
            repair.startupSwitchBoard(s2);
        }
    });
    afterEach(`reset sideEffects`, () => {
        repair.destroy();
        sideEffects.setRepairRate = sinon_1.spy();
        sideEffects.setHeatRate = sinon_1.spy();
        sideEffects.setMaxPower = sinon_1.spy();
    });
    it('exposes all the systems', () => {
        for (let s1 = 0; s1 < empty_epsilon_js_1.ESystem.COUNT; s1++) {
            chai_1.expect(repair.getPrimarySystemStatus(s1).id).to.eql(s1);
        }
        for (let s2 = 0; s2 < model_1.ESwitchBoard.COUNT; s2++) {
            chai_1.expect(repair.getSwitchBoardStatus(s2).id).to.eql(s2);
        }
    });
    it('.startRepairingPrimarySystem(x), .stopRepairingPrimarySystem() affect side effects repair rate and the value of .repairing', () => {
        repair.startRepairingPrimarySystem(empty_epsilon_js_1.ESystem.JumpDrive);
        chai_1.expect(sideEffects.setRepairRate).to.have.been.calledWith(empty_epsilon_js_1.ESystem.JumpDrive, test_kit_1.approx(repair.getPrimarySystemStatus(empty_epsilon_js_1.ESystem.JumpDrive).repairRate));
        chai_1.expect(repair.repairing).to.eql(empty_epsilon_js_1.ESystem.JumpDrive);
        repair.stopRepairingPrimarySystem();
        chai_1.expect(sideEffects.setRepairRate).to.have.been.calledWith(empty_epsilon_js_1.ESystem.JumpDrive, 0);
        chai_1.expect(repair.repairing).to.eql(null);
    });
    it('requires to stop repairing a system before repairing another one', () => {
        repair.startRepairingPrimarySystem(empty_epsilon_js_1.ESystem.JumpDrive);
        chai_1.expect(() => repair.startRepairingPrimarySystem(empty_epsilon_js_1.ESystem.RearShield)).to.throw(Error);
    });
    it('does not throw on repair command for currently repairing system', () => {
        repair.startRepairingPrimarySystem(empty_epsilon_js_1.ESystem.JumpDrive);
        chai_1.expect(() => repair.startRepairingPrimarySystem(empty_epsilon_js_1.ESystem.JumpDrive)).not.to.throw(Error);
    });
    it('A system1 can only be over-powered (more than 100% energy) when all of its supporting system2s are online', () => {
        chai_1.expect(sideEffects.setMaxPower).to.have.been.calledWith(empty_epsilon_js_1.ESystem.Impulse, model_1.PrimarySystem.maxOverPower);
        sideEffects.setMaxPower = sinon_1.spy();
        repair.shutdownSwitchBoard(model_1.ESwitchBoard.A2);
        const dependant = model_1.EcrModel.switchboardstMap[model_1.ESwitchBoard.A2][0];
        chai_1.expect(sideEffects.setMaxPower).to.have.been.calledWith(dependant, sinon_1.match.number.and(sinon_1.match((n) => n <= model_1.PrimarySystem.maxSupportedPower)));
        chai_1.expect(sideEffects.setMaxPower).to.have.not.been.calledWith(dependant, model_1.PrimarySystem.maxOverPower);
    });
    it('When a supporting system2 is offline, the maximum level of a system1’s power will decrease', () => {
        const dependant = model_1.EcrModel.switchboardstMap[model_1.ESwitchBoard.A2][0];
        const dependantStatus = repair.getPrimarySystemStatus(dependant);
        chai_1.expect(dependantStatus.maxPower).to.equal(model_1.PrimarySystem.maxOverPower);
        sideEffects.setMaxPower = sinon_1.spy();
        repair.shutdownSwitchBoard(model_1.ESwitchBoard.A2);
        chai_1.expect(dependantStatus.maxPower).to.be.lessThan(model_1.PrimarySystem.maxSupportedPower);
        chai_1.expect(sideEffects.setMaxPower).to.have.been.calledWith(dependant, test_kit_1.approx(dependantStatus.maxPower));
    });
    describe('When a system1 is over-powered', () => {
        let activeCollectorStatus;
        const dependant = model_1.EcrModel.switchboardstMap[model_1.ESwitchBoard.A2][0];
        beforeEach(() => {
            activeCollectorStatus = repair.getSwitchBoardStatus(model_1.ESwitchBoard.A2);
        });
        it('its supporting system2s accumulate overload', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower });
            chai_1.expect(yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor)).to.eql(0);
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower * 2 });
            chai_1.expect(yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor)).to.be.gt(0);
        })).timeout(10 * 1000);
        it('The overload rate is linear to the level of extra energy', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower * 2 });
            const overPower100Percent = yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor);
            chai_1.expect(overPower100Percent).to.be.greaterThan(0);
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower * 1.5 });
            chai_1.expect(yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor), '1.5 power').to.be.approximately(overPower100Percent * 0.5, overPower100Percent * graceFactor);
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower * 2 });
            yield Promise.resolve();
            chai_1.expect(yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor), '2 power').to.be.approximately(overPower100Percent, overPower100Percent * graceFactor);
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower * 2.5 });
            chai_1.expect(yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor), '2.5 power').to.be.approximately(overPower100Percent * 1.5, overPower100Percent * graceFactor);
            sideEffects.powerInput.next({ system: dependant, power: model_1.PrimarySystem.maxSupportedPower * 3 });
            chai_1.expect(yield test_kit_1.getLinearOverloadDeriviation(activeCollectorStatus, graceFactor), '3 power').to.be.approximately(overPower100Percent * 2, overPower100Percent * graceFactor);
        })).timeout(20 * 1000);
    });
    it('When a system2’s overload level reaches its overload threshold, it goes into error state', () => {
        const system2 = repair.getSwitchBoardStatus(model_1.ESwitchBoard.B2);
        chai_1.expect(system2.isError).to.eql(false);
        repair.addOverloadToSwitchBoard(model_1.ESwitchBoard.B2, system2.overloadErrorThreshold);
        chai_1.expect(system2.isError).to.eql(false);
        repair.addOverloadToSwitchBoard(model_1.ESwitchBoard.B2, 0.00001);
        chai_1.expect(system2.isError).to.eql(true);
    });
    describe('When one or more supporting system2 is in error state', () => {
        const dependant = model_1.EcrModel.switchboardstMap[model_1.ESwitchBoard.A2][0];
        it('the supported system1 begins accumulating heat (the rate does not change according to the amount of systems in error)', () => {
            repair.setError(model_1.ESwitchBoard.A2);
            chai_1.expect(sideEffects.setHeatRate).to.have.been.calledWith(dependant, test_kit_1.approx(model_1.PrimarySystem.heatOnErrorRate));
            sideEffects.setRepairRate = sinon_1.spy();
            repair.setError(model_1.ESwitchBoard.A3);
            chai_1.expect(sideEffects.setHeatRate).to.have.been.calledWith(dependant, test_kit_1.approx(model_1.PrimarySystem.heatOnErrorRate));
            sideEffects.setRepairRate = sinon_1.spy();
            repair.shutdownSwitchBoard(model_1.ESwitchBoard.A2);
            repair.shutdownSwitchBoard(model_1.ESwitchBoard.A3);
            chai_1.expect(sideEffects.setHeatRate).to.have.been.calledWith(dependant, 0);
        });
        it('the repair rate of supported system1 is lower than the normal rate', () => {
            const depRepairRate = repair.getPrimarySystemStatus(dependant).repairRate;
            repair.startRepairingPrimarySystem(dependant);
            repair.setError(model_1.ESwitchBoard.A2);
            chai_1.expect(repair.getPrimarySystemStatus(dependant).repairRate).to.be.lessThan(depRepairRate);
            chai_1.expect(sideEffects.setRepairRate).to.have.been.calledWith(dependant, test_kit_1.approx(repair.getPrimarySystemStatus(dependant).repairRate));
        });
    });
});
//# sourceMappingURL=logic.spec.js.map